#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdlib.h>
#include "timer.h"


#define  TIMER_MAX_SLOTS    ((uint8_t)(10))


Timer::Timer(TC1_t* _timer, TC_CLKSEL_t _clock_prescaler, uint16_t _period, uint8_t _timers, TC_OVFINTLVL_t _int_level)
{
  uint8_t   i;

  if (_timers <= TIMER_MAX_SLOTS) {
    timers = _timers;
  }
  else {
    timers = TIMER_MAX_SLOTS;
  }

  for (i = 0; i < timers; i++) {
    timer_values[i] = 0;
  }

  clock_prescaler = _clock_prescaler;
  change_protect = TIMER_MAX_SLOTS;
  timer = _timer;
  period = _period;

  timer->PER = period;
  timer->INTFLAGS = timer->INTFLAGS;
  timer->INTCTRLA = _int_level;
  timer->CTRLA = clock_prescaler;
}



void Timer::timerSet(uint8_t _timer_slot, uint16_t _time)
{
  if (_timer_slot < timers) {
    change_protect = _timer_slot;
    timer_values[_timer_slot] = _time;
    change_protect = TIMER_MAX_SLOTS;
  }
}



bool Timer::timerGet(uint8_t _timer_slot)
{
  if (timer_values[_timer_slot] != 0) {
    result = false;
  }
  else {
    result = true;
  }

  return (result);
}



void Timer::timerEnable(bool _enable)
{
  if (_enable) {
    timer->CTRLA = clock_prescaler;
  }
  else {
    timer->CTRLA = TC_CLKSEL_OFF_gc;
  }

}



void Timer::handler()
{
  for (timer_index = 0; timer_index < timers; timer_index++)
  {
    if (timer_values[timer_index] != 0)
    {
      if (change_protect != timer_index)
      {
        timer_values[timer_index]--;
      }
    }
  }
}


